/*
Web client

This sketch connects to a website through a MKR NB 1500 board. Specifically,
this example downloads the URL "http://example.org/" and
prints it to the Serial monitor.

Circuit:
    - MKR NB 1500 board
    - Antenna
    - SIM card with a data plan

created 8 Mar 2012
by Tom Igoe
*/

// libraries
#include "ArduinoLowPower.h"
#include <ArduinoJson.h>
#include <MKRNB.h>
#include <SD.h>
#include <SPI.h>
#include <string>
// initialize the library instance
GPRS gprs;
NB nbAccess;

void attachToNetwork();

std::string getResponse(NBClient &client);
std::string getResponseBody(const std::string &response);
void sendData(std::string data);
std::string getResponse(std::string server, std::string path, int port)
{
	NBClient client;
	// if you get a connection, report back via serial:
	if (client.connect(server.c_str(), port)) {
		Serial.println("connected to server");
		// Make a HTTP request:
		client.print("GET ");
		client.print(path.c_str());
		client.println(" HTTP/1.1");
		client.print("Host: ");
		client.println(server.c_str());
		client.println("Connection: close");
		client.println();
	} else {
		// if you didn't get a connection to the server:
		Serial.println("connection failed");
	}

	std::string response = getResponse(client);

	std::string token = getResponseBody(response);
	Serial.println(token.c_str());
	client.stop();
	return token;
}
std::string getResponseBody(const std::string &response)
{
	std::string delimiter = "\r\n\r\n";
	std::string token = response.substr(response.find(delimiter)).replace(0, delimiter.length(), "");
	return token;
}

std::string getResponse(NBClient &client)
{
	String status = client.readStringUntil('\r');
	Serial.println(status);

	std::string response;
	while (client.connected()) {
		if (client.available()) {
			char c = client.read();
			response += c;
		}
	}
	return response;
}

std::string postRequest(std::string server, std::string path, int port, std::string body)
{
	NBClient client;
	// if you get a connection, report back via serial:
	if (client.connect(server.c_str(), port)) {
		Serial.println("connected to server");
		// Make a HTTP request:
		client.print("POST ");
		client.print(path.c_str());
		client.println(" HTTP/1.1");
		client.print("Host: ");
		client.println(server.c_str());
		client.println("Connection: close");
		client.println("Content-Type: application/json");
		client.print("Content-Length: ");
		client.println(body.length());
		client.println();
		client.println(body.c_str());
	} else {
		// if you didn't get a connection to the server:
		Serial.println("connection failed");
	}

	std::string response = getResponse(client);

	std::string token = getResponseBody(response);

	Serial.println(token.c_str());
	client.stop();
	return token;
}

const int wakeUpPin = 0;
const int chipSelect = 6;

void wakeUp()
{
	Serial.println("Wake up");
}
void setup()
{
	// initialize serial communications and wait for port to open:
	Serial.begin(9600);
	Serial1.begin(9600);
	while (!Serial) {
		; // wait for serial port to connect. Needed for native USB port only
	}
	pinMode(wakeUpPin, INPUT);
	// WAKE UP FROM WATCHDOG (PIN)
	//	LowPower.attachInterruptWakeup(wakeUpPin, wakeUp, RISING);

	if (!SD.begin(chipSelect)) {
		Serial.println("initialization failed. Things to check:");
		Serial.println("1. is a card inserted?");
		Serial.println("2. is your wiring correct?");
		Serial.println("3. did you change the chipSelect pin to match your shield or module?");
		Serial.println("Note: press reset button on the board and reopen this Serial Monitor after fixing your issue!");
		while (true)
			;
	}

	Serial.println("Attaching to network");
	attachToNetwork();
}

void attachToNetwork()
{
	Serial.println("Starting Arduino web client.");
	// connection state
	boolean connected = false;

	// After starting the modem with NB.begin()
	// attach to the GPRS network with the APN, login and password
	while (!connected) {
		if ((nbAccess.begin("", "m2m.public.at") == NB_READY) && (gprs.attachGPRS() == GPRS_READY)) {
			connected = true;
		} else {
			Serial.println("Not connected");
			exit(1);
			delay(1000);
		}
	}
}

void loop()
{
	//	LowPower.deepSleep();

	//	Serial.println("Waking up..");

	String dataString;

	while (Serial1.available()) {
		dataString += Serial1.readString();
	}

	// open the file. note that only one file can be open at a time,
	// so you have to close this one before opening another.
	File dataFile = SD.open("datalog.txt", FILE_WRITE);

	// if the file is available, write to it:
	if (dataFile) {
		if (dataString.length() > 0) {
			dataFile.println(dataString);
			Serial.println(dataString);
			sendData(dataString.c_str());
		}
		dataFile.close();
		// print to the serial port too:
		//		Serial.println(dataString);
	}
	// if the file isn't open, pop up an error:
	else {
		Serial.println("error opening datalog.txt");
	}

	delay(100);
	//
	//	Serial.println("Going to sleep");
	//	LowPower.deepSleep(10000);

	// TODO: SAVE TO EEPROM OR SD CARD VIA SPI
	// TODO: REFACTOR CODE
	// TODO: TEST CODE
}
void sendData(std::string data)
{

	Serial.println("connecting...");

	auto response = getResponse("date.jsontest.com", "/", 80);
	//	convert response to json object
	DynamicJsonDocument doc(1024);
	auto error = deserializeJson(doc, response);
	if (error) {
		Serial.print(F("deserializeJson() failed: "));
		Serial.println(error.f_str());
		return;
	}
	const char *date = doc["date"];
	long long milliseconds_since_epoch = doc["milliseconds_since_epoch"].as<long long>();
	const char *time = doc["time"];
	Serial.println(date);
	Serial.println(milliseconds_since_epoch);
	Serial.println(time);
}
