#include "FS.h"
#include "SD.h"
#include "SPI.h"
#include <Arduino.h>
#include <WiFi.h>
#include <esp_now.h>
#include <queue>
#include <sys/unistd.h>

#define RXD2 18
#define TXD2 19

TaskHandle_t Task1;

static std::queue<std::string> queue;
const int wakeUpPin = 4;

[[noreturn]] void Task1code(void *parameter)
{
	Serial.println(xPortGetCoreID());
	while (true) {

		if (!queue.empty()) {
			std::string message = queue.front();
			queue.pop();

			digitalWrite(wakeUpPin, HIGH);
			delay(100);

			Serial.println(message.c_str());
			Serial1.println(message.c_str());

			digitalWrite(wakeUpPin, LOW);
		}

		delay(5000);
		//	Code for task 1 - infinite loop
		//    (...)
	}
}

// SoftwareSerial mySerial(RXD2, TXD2);

enum MessageType{
    dataAck,
    hostChange
};
typedef struct response{
	MessageType type;
	uint8_t mac[6];
	long time;
}response;

void print_mac(const uint8_t *mac){
		for (int i = 0; i < 6; i++) {
		Serial.print(mac[i], HEX);
		Serial.print(":");
	}
	Serial.println();
	Serial.flush();
}

void on_data_sent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
	// go to sleep
}

void on_data_recv(const uint8_t *mac, const uint8_t *incomingData, int len)
{
	// print mac
	Serial.println("Message recieved");
	for (int i = 0; i < 6; i++) {
		Serial.print(mac[i], HEX);
		Serial.print(":");
	}
	Serial.println();
	char data[len];
	memcpy(data, incomingData, len);
	queue.push(data);
}

void setup()
{
	Serial.begin(115200);
	Serial1.begin(9600, SERIAL_8N1, RXD2, TXD2);
	pinMode(wakeUpPin, OUTPUT);
	digitalWrite(wakeUpPin, LOW);

	WiFi.mode(WIFI_STA);
	Serial.println("ESPNow init");
	if (esp_now_init() != ESP_OK) {
		// initialization failed
		Serial.println("ESPNow init failed");
		return; // not sure about this
	}
	Serial.println("ESPNow init success");

	xTaskCreatePinnedToCore(Task1code, /* Function to implement the task */
	                        "Task1",   /* Name of the task */
	                        10000,     /* Stack size in words */
	                        nullptr,   /* Task input parameter */
	                        0,         /* Priority of the task */
	                        &Task1,    /* Task handle. */
	                        0);        /* Core where the task should run */

	esp_now_register_recv_cb(on_data_recv);
	//  write your initialization code here
}

void loop() {}